// Prototipos de rutinas de manipulacin de fechas, diceadas en BORLAND C++ versin 4.0

// Autor:     Antonio Carrillo Ledesma.
// R.F.C.:    CALA-691229-TV5
// Direccin: Amsterdam 312 col. Hipodromo Condesa
// Telfono:  5-74-43-53

// Propiedad intelectual, todos los derechos reservados conforme a la ley, registro en trmite
// Revisin global 1.5-A



#ifndef __FECHAS_HPP__
#define __FECHAS_HPP__


//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Rutinas de manipulacin de fechas. Trabajando como base las conversiones //
// juliana y/o gregoriana, las fecha en forma de cadena estarn de acuerdo  //
// al tipo de formato y delimitador definidos.                              //
//                                                                          //
// La fecha gregoriana tiene por formato:                                   //
//    unsigned int DIA, rango [0 - 31]                                      //
//    unsigned int MES, rango [0 - 12]                                      //
//    unsigned int ANO, rango [1900 - 2100]                                 //
//                                                                          //
// La fecha juliana tiene por formato:                                      //
//    long DIA_JUL, rango [0 - )                                           //
//                                                                          //
// La fecha con formato cadena tiene por formato:                           //
//    Si TIPO_FECHA es (0)  el formato es DD/MM/[CC]YY                      //
//                     (1)  el formato es MM/DD/[CC]YY                      //
//                     (2)  el formato es [CC]YY/MM/DD                      //
//    Si la variable CENTURIA es (1) [CC] existira                          //
//    La variable DELIMIT_FECHA contendra el delimitador de la fecha:       //
//                     Diagonal (/)                                         //
//                     Punto    (.)                                         //
//                     Guion    (-)                                         //
//    Nota: estas variables son definidas en el archivo VAR_GLOB.HPP        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////


void gregorian_julian(const unsigned int dia, const unsigned int mes, const unsigned int ano, unsigned long &dia_jul);
  // De una fecha con formato gregoriano, retorna una fecha con formato juliano
  //     La variable DIA es del rango [0 - 31]
  //                 MES es del rango [0 - 12]
  //                 ANO es del rango [0 - 2100]
  // Nota: En caso de error en la fecha con formato gregoriano retorna cero como dia juliano
  // Nombre del archivo: F_GREJUL.CPP


void julian_gregorian(const unsigned long dia_jul, unsigned int &dia, unsigned int &mes, unsigned int &ano);
  // De una fecha con formato juliano, retorna una fecha con formato gregoriano
  //     La variable DIA es del rango [0 - 31]
  //                 MES es del rango [0 - 12]
  //                 ANO es del rango [0 - 2100]
  // Nota: Esta rutina modifica las variables DIA, MES, ANO en su valor
  //       en caso de error en la fecha con formato juliano retorna cero en las variables DIA, MES, ANO
  // Nombre del archivo: F_JULGRE.CPP

/*
char *f_f_hoy(void);
  // Regresa la fecha actual del sistema con formato cadena como un puntero
  // Nombre del archivo: F_F_HOYP.CPP
*/

void f_f_hoy(char *fecha);
  // Regresa la fecha actual del sistema con formato cadena en la variable FECHA
  // Nota: Esta rutina modifica la variable FECHA en su valor
  // Nombre del archivo: F_F_HOYC.CPP


void f_f_hoy(unsigned long &dia_jul);
  // Regresa la fecha actual del sistema con formato juliano en la variable DIA_JUL
  // Nota: Esta rutina modifica la variable DIA_JUL en su valor
  // Nombre del archivo: F_F_HOYJ.CPP


void f_f_hoy(unsigned int &dia, unsigned int &mes, unsigned int &ano);
  // Regresa la fecha actual del sistema con formato gregoriano en las variables DIA, MES, ANO
  // Nota: Esta rutina modifica las variables DIA, MES, ANO en su valor
  // Nombre del archivo: F_F_HOYG.CPP


unsigned int f_dia_sem(const char *fecha);
  // De una fecha con formato cadena, retorna el numero de da de la semana con rango [0 - 7]
  // (1) Domingo, (2) Lunes, (3) Martes, (4) Miercoles, (5) Jueves,
  // (6) Viernes, (7) Sabado, (0) Error en la fecha juliana
  // Nombre del archivo: F_D_SEMC.CPP


unsigned int f_dia_sem(const unsigned int dia, const unsigned int mes, const unsigned int ano);
  // De una fecha con formato gregoriano, retorna el numero de da de la semana con rango [0 - 7]
  // (1) Domingo, (2) Lunes, (3) Martes, (4) Miercoles, (5) Jueves,
  // (6) Viernes, (7) Sabado, (0) Error en la fecha juliana
  // Nombre del archivo: F_D_SEMG.CPP


unsigned int f_dia_sem(const unsigned long dia_jul);
  // De una fecha con formato juliano, retorna el numero de da de la semana con rango [0 - 7]
  // (1) Domingo, (2) Lunes, (3) Martes, (4) Miercoles, (5) Jueves,
  // (6) Viernes, (7) Sabado, (0) Error en la fecha juliana
  // Nombre del archivo: F_D_SEMJ.CPP




unsigned int f_n_max_dias_mes(const unsigned int mes, const unsigned int ano);
  // Retorna el numero maximo de dias por el mes (MES) y ano (ANO) pasados como parametros
  // Nombre del archivo: F_NMAXDG.CPP


unsigned int f_n_max_dias_mes(const unsigned long dia_jul);
  // Retorna el numero maximo de dias en el mes al que corresponde la fecha con formato juliano
  // Nombre del archivo: F_NMAXDJ.CPP


unsigned int f_n_max_dias_mes(const char *fecha);
  // Retorna el numero maximo de dias en el mes al que corresponde la fecha con formato cadena
  // Nombre del archivo: F_NMAXDC.CPP

unsigned int f_n_dia_mes(const char *fecha);
  // De una fecha con formato cadena, retorna el nmero de das del mes de con rango [0 - 31]
  // en caso de error de la fecha con formato cadena retorna 0.
  // Nombre del archivo: F_N_D_MC.CPP


unsigned int f_n_dia_mes(const unsigned long dia_jul);
  // De una fecha con formato juliano, retorna el nmero de das del mes de con rango [0 - 31]
  // en caso de error de la fecha con formato juliano retorna 0.
  // Nombre del archivo: F_N_D_MJ.CPP


unsigned int f_n_mes(const char *fecha);
  // Regresa el nmero de mes de una fecha en formato de cadena, retornando sta
  // como un entero del rango [0 - 12]
  // (1) Enero, (2) Febrero, (3) Marzo,      (4) Abril,    (5) Mayo,       (6) Junio
  // (7) Julio, (8) Agosto,  (9) Septiembre, (10) Octubre, (11) Noviembre, (12) Diciembre
  // (0) Error en el formato de la fecha
  // Nombre del archivo: F_N_MESC.CPP


unsigned int f_n_mes(const unsigned long dia_jul);
  // Regresa el nmero de mes de una fecha con formato juliano, retornando sta
  // como un entero del rango [0 - 12]
  // (1) Enero, (2) Febrero, (3) Marzo,      (4) Abril,    (5) Mayo,       (6) Junio
  // (7) Julio, (8) Agosto,  (9) Septiembre, (10) Octubre, (11) Noviembre, (12) Diciembre
  // (0) Error en el formato de la fecha
  // Nombre del archivo: F_N_MESJ.CPP


unsigned int f_n_ano(const char *fecha);
  // Regresa el nmero de ao en una fecha en formato cadena, retornando sta
  // como un entero del rango [0 - 2100]
  // (0) Error en el formato de la fecha
  // Nombre del archivo: F_N_ANOC.CPP


unsigned int f_n_ano(const unsigned long dia_jul);
  // Regresa el nmero de ao en una fecha con formato juliano, retornando esta
  // como un entero del rango [0 - 2100]
  // (0) Error en el formato de la fecha
  // Nombre del archivo: F_N_ANOJ.CPP


void f_dia_let(const char *fecha, char *d_let, const unsigned int t_l = 1);
  // Regresa el nombre del da de la semana en letra, apartir de una fecha con formato cadena
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_DIALEC.CPP


void f_dia_let(const unsigned int n_dia, char *d_let, const unsigned int t_l = 1);
  // Regresa el nombre del da de la semana en letra, apartir de el numero de dia
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_DIALEG.CPP


void f_dia_let(const unsigned long dia_jul, char *d_let, const unsigned int t_l = 1);
  // Regresa el nombre del da de la semana en letra, apartir de una fecha con formato juliano
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_DIALEJ.CPP


void f_mes_let(const char *fecha, char *m_let, const unsigned int t_l = 1);
  // Regresa el nombre del mes en letra, apartir de una fecha con formato cadena
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_MESLEC.CPP


void f_mes_let(const unsigned int mes, char *m_let, const unsigned int t_l = 1);
  // Regresa el nombre del mes en letra, apartir de el numero de mes
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_MESLEG.CPP


void f_mes_let(const unsigned long dia_jul, char *m_let, const unsigned int t_l = 1);
  // Regresa el nombre del mes en letra, apartir de una fecha con formato juliano
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_MESLEJ.CPP


void f_fecha__greg(const char *fecha, unsigned int &dia, unsigned int &mes, unsigned int &ano);
  // De una fecha con formato cadena, retorna el nmero de Das, Meses, y Aos
  //  Esta rutina modifica las variables DIA, MES, ANO en su valor
  //  En caso de error en la fecha retornar (0) en las variables DIA, MES, ANO
  // Nombre del archivo: F_FECGRE.CPP


void f_fecha__juli(const char *fecha, unsigned long &dia_jul);
  // De una fecha con formato cadena, retorna una fecha con formato juliano
  // En caso de error en la fecha retornar (0) en la variable DIA_JUL
  // Nombre del archivo: F_FECJUL.CPP


void f_greg__fecha(const unsigned int dia, const unsigned int mes, const unsigned int ano, char *fecha);
  // De una fecha con formato gregoriano, retorna una fecha con formato cadena
  // En caso de error en la fecha retornar una cadena en blanco
  // Nombre del archivo: F_GREFEC.CPP


void f_juli__fecha(const unsigned long dia_jul, char *fecha);
  // De una fecha con formato juliano, regresa una fecha con formato cadena
  // En caso de error en la fecha retornar una cadena en blanco
  // Nombre del archivo: F_JULFEC.CPP


unsigned int f_val_fecha(const char *fecha);
  // Valida una fecha con formato cadena, retornando:
  //     (0) Sin errores
  //     (1) Error en los Das
  //     (2) Error en los Mes
  //     (3) Error en los Aos
  // Nombre del archivo: F_VALFEC.CPP


unsigned int f_val_fecha(const unsigned int dia, const unsigned int mes, const unsigned int ano);
  // Valida una fecha con formato gregoriano, retornando:
  //     (0) Sin errores
  //     (1) Error en los Das
  //     (2) Error en los Mes
  //     (3) Error en los Aos
  // Nombre del archivo: F_VALFEG.CPP

/*
char *f_quitar_delim(const char *fecha);
  // De una fecha con formato de cadena, retorna una fecha con formato CCYYMMDD
  // En caso de error retorna "00000000" en forma de cadena
  // Nombre del archivo: F_QUIDEP.CPP
*/

void f_quitar_delim(const char *fecha, char *fecha_s_d);
  // De una fecha con formato de cadena, retorna una fecha con formato CCYYMMDD
  // En caso de error retorna "00000000" en forma de cadena
  // Nombre del archivo: F_QUIDEC.CPP


void f_quitar_delim(const unsigned int dia, const unsigned int mes, const unsigned int ano, char *fecha_s_d);
  // De una fecha con formato de cadena, retorna una fecha con formato CCYYMMDD
  // En caso de error retorna "00000000" en forma de cadena
  // Nombre del archivo: F_QUIDEG.CPP


void f_quitar_delim(const unsigned long dia_jul, char *fecha_s_d);
  // De una fecha con formato de cadena, retorna una fecha con formato CCYYMMDD
  // En caso de error retorna "00000000" en forma de cadena
  // Nombre del archivo: F_QUIDEJ.CPP


void f_poner_delim(const char *fecha, char *fecha_cd);
  // De una fecha del tipo CCYYMMDD, retorna una fecha en formato cadena
  // Nombre del archivo: F_PODELC.CPP


void f_poner_delim(const unsigned int dia, const unsigned int mes, const unsigned int ano, char *fecha_cd);
  // De una fecha con formato gregoriano, retorna una fecha con formato cadena
  // Nombre del archivo: F_PODELG.CPP


void f_poner_delim(const unsigned long dia_jul, char *fecha_cd);
  // De una fecha con formato juliano, retorna una fecha con formato cadena
  // Nombre del archivo: F_PODELJ.CPP

/*
char *f_poner_delim(const char *fecha);
  // De una fecha del tipo CCYYMMDD, retorna una fecha en formato cadena
  // Nombre del archivo: F_PODELP.CPP
*/


void f_f__ddmmmaa(const char *fecha, const unsigned int t_l, char *xfecha);
  // De una fecha con formato cadena, retorna una cadena del tipo DD/MMM/CCYY,
  // el mes (MMM) ser en letra
  //     Si T_L es (0) ser en ingles
  //               (1) ser en espaol
  // Si hay error en la fecha retornar una cadena de espacios
  // Nombre del archivo: F_MES3LE.CPP


void f_fecha_escrita(const char *fecha, char *fec_esc, const unsigned int t_p, const unsigned int id = 1);
  // De una fecha con formato cadena, retorna la fecha escrita con el sigiente formato:
  // Si T_P es (0) el formato es:
  //    Si ID es (1) el formato es [DD DE MMMMMMMM DE CCAA] en espaol
  //    Si ID es (0) el formato es [DD OF MMMMMMMM OF CCAA] en ingles
  // Si T_P es (1) el formato es:
  //    Si ID es (1) el formato es [DDDDDDDD DD DE MMMMMMMM DE CCAA] en espaol
  //    Si ID es (0) el formato es [DDDDDDDD DD OF MMMMMMMM OF CCAA] en ingles
  // Nombre del archivo: F_FECESC.CPP


void f_fecha_escrita(const unsigned int dia, const unsigned int mes, const unsigned int ano, char *fec_esc, const unsigned int t_p = 1);
  // De una fecha con formato gregoriano, retorna la fecha escrita con el sigiente formato:
  // Si T_P es (1) el formato es [DD DE MMMMMMMM DE CCAA] en espaol
  // Si T_P es (0) el formato es [DD OF MMMMMMMM OF CCAA] en ingles
  // Nombre del archivo: F_FECESG.CPP


void f_fecha_escrita(const unsigned long dia_jul, char *fec_esc, const unsigned int t_p = 1);
  // De una fecha con formato juliano, retorna la fecha escrita con el sigiente formato:
  // Si T_P es (1) el formato es [DD DE MMMMMMMM DE CCAA] en espaol
  // Si T_P es (0) el formato es [DD OF MMMMMMMM OF CCAA] en ingles
  // Nombre del archivo: F_FECESJ.CPP


void cal_horas__fec_hor(const char *cf1, const char *ch1, const char *cf2, const char *ch2, int &hor, int &min);
  // Retorna la diferencia en horas y minutos de un par de fechas con la hora corespondiente
  // Retorna -1 en HOR y MIN si existe alguna inconsistencia en fechas o horas
  // Nota: Los segundos no son tomados en cuenta
  // Nombre del archivo: F_CALHOR.CPP


#endif 
