// Autor:     Antonio Carrillo Ledesma.
// R.F.C.:    CALA-691229-TV5
// Direccin: Amsterdam 312 col. Hipodromo Condesa
// Telfono:  5-74-43-53

// Propiedad intelectual, todos los derechos reservados conforme a la ley, registro en trmite
// Revisin  1.0-A

////////////////////////////////////////////////////////////////
// Falta controlar si es posible escribir en la unidad indicada
////////////////////////////////////////////////////////////////


#include "man_arch.hpp"
#include "r_esp.hpp"
#include "r_var.hpp"
#include "nucleo.hpp"


extern "C" {
   #include <dir.h>
   #include <dos.h>
   #include <string.h>
   #include <stdlib.h>
}

// Objeto global al nuclo del sistema
extern Nucleo *NCO;

#define TABULADOR 8

// Posibles estados del manipulador de archivos
// (99)  Fin de archivo
// (0)   Normal



// constructor del manipulador de archivos
Manipulador_archivos::Manipulador_archivos(void)
{
   ESTADO = 0;
   DELIMITADOR = new char[4];
   DELIMITADOR[0] = 13;
   DELIMITADOR[1] = 10;
   DELIMITADOR[2] = 0;
}


// Destructor del manipulador de archivos
Manipulador_archivos::~Manipulador_archivos()
{
   // Cierra el archivo de trabajo
   fclose(ARCHIVO_TRABAJO);
   delete []DELIMITADOR;
}


// constructor del manipulador de archivos
void Manipulador_archivos::Parametros(const char *archivo, const int modo, const unsigned int lg_max, const char *rutina)
{
   MODO = modo;
   LG_MAX = lg_max;

   ARCHIVO = archivo;
   RUTINA = rutina;

   // Establece el tipo de atributo para apertura del archivo
   char *ATRIBUTO;
   if(MODO == LEE_ARCHIVO)      ATRIBUTO = "rb";
   if(MODO == GRABA_ARCHIVO)    ATRIBUTO = "wb";
   if(MODO == ADICIONA_ARCHIVO) ATRIBUTO = "ab";
   // Si no encuentra el archivo y el modo es de lectura manda mesage de error
   if(MODO == LEE_ARCHIVO) {
      if(!n_archivos_directorios(ARCHIVO,_A_ARCH)) NCO->Ctrl_error(-6,ARCHIVO,RUTINA);
    } else {
      // Controlar si el drive puede ser escrito
//      if() NCO->Ctrl_error(-6,ARCHIVO,RUTINA);
   }
   // Apertura del archivo
   ARCHIVO_TRABAJO = fopen(ARCHIVO,ATRIBUTO);
   if(!ARCHIVO_TRABAJO) NCO->Ctrl_error(-4,ARCHIVO,RUTINA);
}



// Cuenta el numero de caracteres y lineas dentro del archivo si T_P es 1
// Si T_P es cero retorna el numero de caracteres reales
void Manipulador_archivos::Longitud_archivo(long &lg_archivo, long &nm_lineas, const int t_p)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   lg_archivo = nm_lineas = ESTADO = 0;
   rewind(ARCHIVO_TRABAJO);
   while(!feof(ARCHIVO_TRABAJO)) {
      car = fgetc(ARCHIVO_TRABAJO);
      if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
      if(t_p) {
         if(!car || car == 7 || car == 13 || car == 255) continue;
         if(car == 9) {
            lg_archivo += TABULADOR;
            continue;
         }
         if(car == 10) nm_lineas ++;
      }
      lg_archivo++;
   }
   rewind(ARCHIVO_TRABAJO);
}



// Lee la siguiente linea del archivo, retornando la longitud de la linea
long Manipulador_archivos::Lee_linea(char *cadena)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   char *pcad = cadena;
   longtmp = 0;
   while(!feof(ARCHIVO_TRABAJO)) {
      car = fgetc(ARCHIVO_TRABAJO);
      if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
      if(car == 0 || car == 7 || car == 13 || car == 255) continue;
      if(car == 10 || longtmp >= LG_MAX) break;
      if(car == 9) {
         if(longtmp + TABULADOR < LG_MAX) {
            for(int i = 0; i < TABULADOR; i++) *pcad = 32, pcad++;
            longtmp += TABULADOR;
         }
         continue;
      }
      *pcad = car, pcad++, longtmp++;
   }
   *pcad = 0;
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   return longtmp;
}


// Graba una cadena como una linea del archivo, retornando la longitud de esta
long Manipulador_archivos::Graba_linea(const char *cadena)
{
   inttmp = strlen(cadena);
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   if(inttmp) longtmp = fwrite(cadena,1,inttmp,ARCHIVO_TRABAJO);
   fwrite(DELIMITADOR,1,2,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
   return longtmp;
}

// Busca un numero de linea determinado en el archivo
long Manipulador_archivos::Busca_linea(const long linea)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   longtmp = 1;
   rewind(ARCHIVO_TRABAJO);
   if(linea < 2) return 1;
   while(!feof(ARCHIVO_TRABAJO)) {
      car = fgetc(ARCHIVO_TRABAJO);
      if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
      if(car == 10) {
         if(longtmp >= linea) break;
          else longtmp++;
      }
   }
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   return longtmp;
}




// Lee el numero de caracteres indicados en N_C y los deja en la cadena CADENA, retornando  el numero de caracteres leidos
long Manipulador_archivos::Lee_caracteres(char *cadena, const unsigned n_c)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   longtmp = fread(cadena,1,(unsigned int) n_c,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   return longtmp;
}


// Graba una una cadena de caracteres, retornando el numero de caracteres grabados
long Manipulador_archivos::Graba_caracteres(const char *cadena, const unsigned n_c)
{
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   longtmp = fwrite(cadena,1,(unsigned int) n_c,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
   return longtmp;
}


// Lee un entero
void Manipulador_archivos::Lee_entero(int &num)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fread(&inttmp,2,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   num = inttmp;
}

// Lee un entero sin signo
void Manipulador_archivos::Lee_entero(unsigned int &num)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fread(&uinttmp,2,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   num = uinttmp;
}

// Lee un caracter sin signo
void Manipulador_archivos::Lee_caracter(char &car)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fread(&cartmp,1,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   car = cartmp;
}


// Lee un entero largo
void Manipulador_archivos::Lee_entero_largo(long &num)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fread(&longtmp,4,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo
   num = longtmp;
}

// Lee un flotante double
void Manipulador_archivos::Lee_double(double &num)
{
   #ifdef __VALIDA__
   if(MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fread(&num,8,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-5,ARCHIVO,RUTINA);
   if(feof(ARCHIVO_TRABAJO)) ESTADO = 99;  // (99) Fin de archivo

}


// Graba un entero
void Manipulador_archivos::Graba_entero(const int ent)
{
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fwrite(&ent,2,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
}

// Graba un entero largo
void Manipulador_archivos::Graba_entero_largo(const long int ent)
{
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fwrite(&ent,4,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
}


// Graba un entero sin signo
void Manipulador_archivos::Graba_entero(const unsigned int ent)
{
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fwrite(&ent,2,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
}

// Graba caracter sin signo
void Manipulador_archivos::Graba_caracter(const char car)
{
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fwrite(&car,1,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
}


// Graba un flotante double 
void Manipulador_archivos::Graba_double(const double num)
{
   #ifdef __VALIDA__
   if(!MODO) NCO->Ctrl_error(-15,ARCHIVO,RUTINA);
   #endif
   fwrite(&num,8,1,ARCHIVO_TRABAJO);
   if(ferror(ARCHIVO_TRABAJO)) NCO->Ctrl_error(-3,ARCHIVO,RUTINA);
}

