// Autor:     Antonio Carrillo Ledesma.
// R.F.C.:    CALA-691229-TV5
// Direccin: Amsterdam 312 col. Hipodromo Condesa
// Telfono:  5-74-43-53

// Propiedad intelectual, todos los derechos reservados conforme a la ley, registro en trmite
// Revisin  1.1-A



#include "v_grafic.hpp"
#include "r_var.hpp"
#include "nucleo.hpp"
#include "teclado.hpp"

extern "C" {
   #include <graphics.h>
   #include <math.h>
   #include <string.h>
}


// Definicion externa del objeto Nucleo
extern Nucleo *NCO;


/////////////////////////////////////////////////////////////////////
//                   Ventana de Graficacin                        //
/////////////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////
// Inicializa la clase                                     //
/////////////////////////////////////////////////////////////
void Ventana_graficacion::Inicializa(const int t_proy, const double factor) const
{

   // Fija la posicion dentro de los ejes (real)
   ((Ventana_graficacion*)this)->C_reales.x = ((Ventana_graficacion*)this)->C_reales.y = ((Ventana_graficacion*)this)->C_reales.z = 0.0;

   // Fija la dimenciones de los ejes (real)
   ((Ventana_graficacion*)this)->Maximo.x = ((Ventana_graficacion*)this)->Maximo.y = ((Ventana_graficacion*)this)->Maximo.z = 10;
   ((Ventana_graficacion*)this)->Minimo.x = ((Ventana_graficacion*)this)->Minimo.y = ((Ventana_graficacion*)this)->Minimo.z = -10;
   ((Ventana_graficacion*)this)->Factor = factor;

   // Ejes en tres dimenciones
   ((Ventana_graficacion*)this)->Proyeccion = t_proy;
   ((Ventana_graficacion*)this)->TETA = M_PI / 13, ((Ventana_graficacion*)this)->FI = 9.0 * M_PI / 20.0;
   // Fija las escalas
   Fija_escalas();
   Calcula_valores();
}


//////////////////////////////////////////////////////////////////
//             Fija la escala dentro de la ventana              //
//////////////////////////////////////////////////////////////////

void Ventana_graficacion::Fija_escalas(void) const
{
   // Valores numericos para las proyecciones
   ((Ventana_graficacion*)this)->Centro.x   = X1 + Retorna_inicio_X() + ((Largo_X() - Retorna_inicio_X()) / 2.0);
   ((Ventana_graficacion*)this)->Centro.y   = Y1 + Retorna_inicio_Y() + ((Largo_Y() - Retorna_inicio_Y()) / 2.0);
   C_3D xesc;
   xesc.x = Maximo.x > -Minimo.x ? Maximo.x * 2: Minimo.x * (-2);
   xesc.y = Maximo.y > -Minimo.y ? Maximo.y * 2: Minimo.y * (-2);
   xesc.z = Maximo.z > -Minimo.z ? Maximo.z * 2: Minimo.z * (-2);
   int min = 0;
   // Tomo las escalas con referencia al tamao del lado mas pequeo de la ventana
   min = (Largo_X() - Retorna_inicio_X()) < (Largo_Y() - Retorna_inicio_Y()) ? (Largo_X() - Retorna_inicio_X()) : (Largo_Y() - Retorna_inicio_Y()); 
   min *= Factor;
   ((Ventana_graficacion*)this)->Escala.x = (xesc.x == 0.0 ? 0.0 : min / xesc.x);
   ((Ventana_graficacion*)this)->Escala.y = (xesc.y == 0.0 ? 0.0 : min / xesc.y);
   ((Ventana_graficacion*)this)->Escala.z = (xesc.z == 0.0 ? 0.0 : min / xesc.z);
}


/////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////
void Ventana_graficacion::Calcula_valores(void) const
{
   ((Ventana_graficacion*)this)->ST = sin(TETA);
   ((Ventana_graficacion*)this)->CT = cos(TETA);
   ((Ventana_graficacion*)this)->SF = sin(FI);
   ((Ventana_graficacion*)this)->CF = cos(FI);
   ((Ventana_graficacion*)this)->x1 = CT * CF, ((Ventana_graficacion*)this)->x2 = ST * CF, ((Ventana_graficacion*)this)->x3 = CT * SF, ((Ventana_graficacion*)this)->x4 = ST * SF;
}


/////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////
void Ventana_graficacion::Acercamiento(void) const
{
/*
   double xx;
   C_2I v_max, v_min;
   C_dbl vent;
   if(Selecciona_porcion_ventana(vent)) {
      v_max.x = (X1 + vent.x1) - Centro.x;
      v_max.y = Centro.y - (vent.y1 + Y1);
      v_min.x = (vent.x2 + X1) - Centro.x;
      v_min.y = Centro.y - (vent.y2 + Y1);

      C_3D xv_max, xv_min;

      xv_max.x = (double) v_max.x / Escala.x;
      xv_min.x = (double) v_min.x / Escala.x;
      xv_max.y = (double) v_max.y / Escala.y;
      xv_min.y = (double) v_min.y / Escala.y;

      if(xv_max.x < xv_min.x)  xx = xv_max.x, xv_max.x = xv_min.x, xv_min.x = xx;
      if(xv_max.y < xv_min.y)  xx = xv_max.y, xv_max.y = xv_min.y, xv_min.y = xx;

      double dif = fabs(xv_max.x - xv_min.x) > fabs(xv_max.y - xv_min.y) ? fabs(xv_max.x - xv_min.x) : fabs(xv_max.y - xv_min.y);
      xv_min.x = xv_max.x - dif;                                       
      xv_min.y = xv_max.y - dif;                                       

      Fija_dimenciones(xv_max,xv_min);
   }
*/
}

/////////////////////////////////////////////////////////////
// Retorna la posicion del mouse dentro de la ventana de   //
// graficacion                                             //
/////////////////////////////////////////////////////////////
int Ventana_graficacion::Posicion_mouse(C_2D &pos) const
{
   int x, y;
   Ventana::Posicion_mouse(x,y);
   pos.x = ((double) ((X1 + x) - Centro.x) / Escala.x);
   pos.y = ((double) (Centro.y - (y + Y1)) / Escala.y);
   return MOUSE_DENTRO;
}




/////////////////////////////////////////////////////////////
// Mueve el origen del eje coordenado                      //
/////////////////////////////////////////////////////////////
void Ventana_graficacion::Mover_origen(const int tp) const
{
   switch(tp) {
     case FLECHA_ARRIBA:    // Hacia arriba
       if(Centro.y > Y1 + 3) ((Ventana_graficacion*)this)->Centro.y--;
       if(Revisa_graficacion()) ((Ventana_graficacion*)this)->Centro.y++;;
       break;
     case FLECHA_ABAJO:     // Hacia abajo
       if(Centro.y < Y2) ((Ventana_graficacion*)this)->Centro.y++;
       if(Revisa_graficacion()) ((Ventana_graficacion*)this)->Centro.y--;;
       break;
     case FLECHA_IZQUIERDA: // Hacia la izquierda
       if(Centro.x > X1) ((Ventana_graficacion*)this)->Centro.x--;
       if(Revisa_graficacion()) ((Ventana_graficacion*)this)->Centro.x++;;
       break;
     case FLECHA_DERECHA:   // hacia la derecha
       if(Centro.x < X2) ((Ventana_graficacion*)this)->Centro.x++;
       if(Revisa_graficacion()) ((Ventana_graficacion*)this)->Centro.x--;;
       break;
   }
}


/////////////////////////////////////////////////////////////
// Acercar / alejer la ventana                             //
/////////////////////////////////////////////////////////////
void Ventana_graficacion::Acercar_alejar(const char tp) const
{
   switch(tp) {
      case '+': // Acerca los ejes coordenados
        ((Ventana_graficacion*)this)->Escala.x *= 1.01, ((Ventana_graficacion*)this)->Escala.y *= 1.01, ((Ventana_graficacion*)this)->Escala.z *= 1.01; 
        if(!Revisa_graficacion()) break;
      case '-': // Aleja los ejes coordenados
        ((Ventana_graficacion*)this)->Escala.x *= 0.99, ((Ventana_graficacion*)this)->Escala.y *= 0.99, ((Ventana_graficacion*)this)->Escala.z *= 0.99; 
        break;
   }
}






/////////////////////////////////////////////////////////////
//                                                         //
//        Ventana de graficacion en 3 dimenciones          //
//                                                         //
/////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////
// Rota ejes coordenados                                   //
/////////////////////////////////////////////////////////////
void Ventana_graficacion_3D::Rota_ejes(const int tp) const
{
   int ang = 0;
   double rad = 0;
   switch(tp) {
      case PGDN:
      case PGUP:
      case FLECHA_ARRIBA:     // Rota hacia arriba
        ang = 1, rad = 0.005;
        break;
      case FLECHA_ABAJO:      // Rota hacia abajo
        ang = 1, rad = -0.005;
        break;
      case FLECHA_IZQUIERDA:  // Rota hacia la izquierda
        rad = 0.01;
        break;
      case FLECHA_DERECHA:    // Rota hacia la derecha
        rad = -0.01;
   }
   if(ang) ((Ventana_graficacion_3D*)this)->FI += rad;
    else ((Ventana_graficacion_3D*)this)->TETA += rad;
   Calcula_valores();
}





///////////////////////////////////////////////////////////////
//                                                           //
//  Ventana con cursor                                       //
//                                                           //
///////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// Inicializa la clase                                     //
/////////////////////////////////////////////////////////////
void V_cursor::Inicializa(void) const
{
   // Define al puntor que contiene la imagen debajo del cursor
   unsigned int i = imagesize(0,0,7,7);
   ((V_cursor*)this)->Cursor = new char [i+100];
   // Fija la posicion del cursor
   ((V_cursor*)this)->Pos_ant.x = ((V_cursor*)this)->Pos_ant.y = 0;
   sw_cursor = 0;
}



/////////////////////////////////////////////////////////////
// Controla la visualizacion del cursor                    //
/////////////////////////////////////////////////////////////
void V_cursor::Visualiza_cursor(const int tp) const
{
   // Hace no visible el mouse
   NCO->Muestra_mouse(NO);
   if(tp) {
      if(Proyeccion) {
         if(Proyecciones_2D(C_reales,((V_cursor*)this)->Pos_ant)) return;
       } else {
         if(Proyecciones_3D(C_reales,((V_cursor*)this)->Pos_ant)) return;
      }
      getimage(Pos_ant.x,Pos_ant.y,Pos_ant.x+7,Pos_ant.y+7,Cursor);
      setcolor(Magenta1);
      line(Pos_ant.x,Pos_ant.y,Pos_ant.x+7,Pos_ant.y+7);
      line(Pos_ant.x,Pos_ant.y,Pos_ant.x+5,Pos_ant.y);
      line(Pos_ant.x,Pos_ant.y,Pos_ant.x,Pos_ant.y+5);
      sw_cursor = 1;
   } else {
      if(sw_cursor) putimage(Pos_ant.x,Pos_ant.y,Cursor,COPY_PUT);
      sw_cursor = 0;
   }
}





/////////////////////////////////////////////////////////////
//                                                         //
// Ventana con ejes coordenados en 2D y 3D                 //
//                                                         //
/////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////
// Inicializa la clase                                     //
/////////////////////////////////////////////////////////////
void V_ejes_coordenados::Inicializa(const int t_proy, const double factor, const char *n_ejes) const
{
   // Inicializa la ventana de graficacion en 3D
   Ventana_graficacion_3D::Inicializa(t_proy,factor);
   // Indica que visualize el nombre de los ejes
   Visualiza_nombre_eje = 1;
   // Selecciona el tipo de ejes coordenados a visualizar
   strcpy(((V_ejes_coordenados*)this)->Nombre_ejes,n_ejes);
   // Fija el color de los ejes coordenados
   ((V_ejes_coordenados*)this)->Color_eje.x = Rojo1;      // (0) Eje X
   ((V_ejes_coordenados*)this)->Color_eje.y = Verde9;     // (1) Eje Y
   ((V_ejes_coordenados*)this)->Color_eje.z = Azul1;      // (2) Eje Z
}


/////////////////////////////////////////////////////////////
// Dibuja los ejes coordenados en 2D                       //
/////////////////////////////////////////////////////////////
void V_ejes_coordenados::Dibuja_ejes_2D(const int tp) const
{
   char xcad[4];
   NCO->Muestra_mouse(NO);
   int cl,i, sw = 0;
   C_3D pos, ctr;
   C_2I pt, ct;
   pos.x = 0, pos.y = 0, pos.z = 0;
   ctr = pos;
   // Eje X
   Proyecciones_2D(pos,ct);
   for(i = 0; i < 2;++i) {
      setlinestyle(1-i,0,i);
      if(i) pos.x = Maximo.x;
       else pos.x = Minimo.x;
      if(tp) cl = Color_eje.x;
       else cl = Actual_color_fondo();
      Dibuja_linea_2D(ctr,pos,cl);
      if(Visualiza_nombre_eje) {
         sw = 0;
         if(!Proyecciones_2D(pos,pt)) sw = 1;
         if(sw) {
            // Indica el nombre del eje coordenado
            if(!i) {
               xcad[0] = '-',xcad[1] = Nombre_ejes[0], xcad[2] = 0;
               outtextxy(pt.x-20,pt.y-5,xcad);
             } else {
               xcad[0] = Nombre_ejes[0], xcad[1] = 0;
               outtextxy(pt.x+8,pt.y-5,xcad);
            }
         }
      }
      pos.x = 0.0;
      // Eje Y
      if(i) pos.y = Maximo.y;
       else pos.y = Minimo.y;
      Proyecciones_2D(pos,pt);
      if(tp) cl = Color_eje.y;
       else cl = Actual_color_fondo();
      Dibuja_linea_2D(ctr,pos,cl);
      // Indica el nombre del eje coordenado
      if(Visualiza_nombre_eje) {
         sw = 0;
         if(!Proyecciones_2D(pos,pt)) sw = 1;
         if(sw) {
            if(!i) {
               xcad[0] = '-',xcad[1] = Nombre_ejes[1], xcad[2] = 0;
               outtextxy(pt.x-20,pt.y+5,xcad);
             } else {
               xcad[0] = Nombre_ejes[1], xcad[1] = 0;
               outtextxy(pt.x+5,pt.y-13,xcad);
            }
         }
      }
      pos.y = 0.0;
   }
}


/////////////////////////////////////////////////////////////
// Dibuja los ejes coordenados en 3D                       //
/////////////////////////////////////////////////////////////
void V_ejes_coordenados::Dibuja_ejes_3D(const int tp) const
{
   char xcad[4];
   NCO->Muestra_mouse(NO);
   int cl,i, sw = 0;
   C_3D pos, ctr;
   C_2I pt, ct;
   pos.x = 0, pos.y = 0, pos.z = 0;
   ctr = pos;
   // Eje X
   Proyecciones_3D(pos,ct);
   for(i = 0; i < 2;++i) {
      setlinestyle(1-i,0,i);
      if(i) pos.x = Maximo.x;
       else pos.x = Minimo.x;
      if(tp) cl = Color_eje.x;
       else cl = Actual_color_fondo();
      if(Proyeccion) Dibuja_linea_2D(ctr,pos,cl);
       else Dibuja_linea_3D(ctr,pos,cl);
      if(Visualiza_nombre_eje) {
         sw = 0;
         if(!Proyecciones_3D(pos,pt)) sw = 1;
         if(sw) {
            // Indica el nombre del eje coordenado
            if(!i) {
               xcad[0] = '-',xcad[1] = Nombre_ejes[0], xcad[2] = 0;
               outtextxy(pt.x-20,pt.y-5,xcad);
             } else {
               xcad[0] = Nombre_ejes[0], xcad[1] = 0;
               outtextxy(pt.x+8,pt.y-5,xcad);
            }
         }
      }
      pos.x = 0.0;
      // Eje Y
      if(i) pos.y = Maximo.y;
       else pos.y = Minimo.y;
      Proyecciones_3D(pos,pt);
      if(tp) cl = Color_eje.y;
       else cl = Actual_color_fondo();
      Dibuja_linea_3D(ctr,pos,cl);
      // Indica el nombre del eje coordenado
      if(Visualiza_nombre_eje) {
         sw = 0;
         if(!Proyecciones_3D(pos,pt)) sw = 1;
         if(sw) {
            if(!i) {
               xcad[0] = '-',xcad[1] = Nombre_ejes[1], xcad[2] = 0;
               outtextxy(pt.x-20,pt.y+5,xcad);
             } else {
               xcad[0] = Nombre_ejes[1], xcad[1] = 0;
               outtextxy(pt.x+5,pt.y-13,xcad);
            }
         }
      }
      pos.y = 0.0;
      // Eje Z
      if(i) pos.z = Maximo.z;
       else pos.z = Minimo.z;
      Proyecciones_3D(pos,pt);
      if(tp) cl = Color_eje.z;
       else cl = Actual_color_fondo();
      Dibuja_linea_3D(ctr,pos,cl);
      // Indica el nombre del eje coordenado
      if(Visualiza_nombre_eje) {
         sw = 0;
         if(!Proyecciones_3D(pos,pt)) sw = 1;
         if(sw) {
            if(!i) {
               xcad[0] = '-',xcad[1] = Nombre_ejes[2], xcad[2] = 0;
               outtextxy(pt.x-20,pt.y+5,xcad);
             } else {
               xcad[0] = Nombre_ejes[2], xcad[1] = 0;
               outtextxy(pt.x+8,pt.y-13,xcad);
            }
         }
      }
      pos.z = 0.0;
   }
}





///////////////////////////////////////////////////////////////
//                                                           //
// Ventana de graficacion en 2D y 3D con cursor y ejes       //
// cordenados                                                //
//                                                           //
///////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////
// Revisa que la zona de graficacion este dentro de la     //
// ventana                                                 //
/////////////////////////////////////////////////////////////

int V_graficacion::Revisa_graficacion(void) const
{
   C_3D pt_r;
   C_2I pt_v;
   int i;
   double xdim[6][3];
   for(i = 0; i < 6; i++) xdim[i][0] = xdim[i][1] = xdim[i][2] = 0;
   xdim[0][0] = Maximo.x * 1.14, xdim[1][1] = Maximo.y * 1.14, xdim[2][2] = Maximo.z * 1.14;
   xdim[3][0] = Minimo.x * 1.18, xdim[4][1] = Minimo.y * 1.18, xdim[5][2] = Minimo.z * 1.18;
   for(i = 0; i < 6; i++) {
      pt_r.x = xdim[i][0], pt_r.y = xdim[i][1], pt_r.z = xdim[i][2];
      if(Proyeccion) {
         if(Proyecciones_2D(pt_r, pt_v)) return 1;
       } else {
         if(Proyecciones_3D(pt_r, pt_v)) return 1; 
      }
   }
   return 0;
}       


