// Autor:     Antonio Carrillo Ledesma.
// R.F.C.:    CALA-691229-TV5
// Direccin: Amsterdam 312 col. Hipodromo Condesa
// Telfono:  5-74-43-53

// Propiedad intelectual, todos los derechos reservados conforme a la ley, registro en trmite
// Revisin  1.1-A


#include "ana_exp.hpp"
#include "r_var.hpp"
#include "cad_car.hpp"
#include "vis_ayud.hpp"
#include "pila.hpp"



extern "C" {
   #include <string.h>
   #include <stdlib.h>
   #include <stdio.h>
   #include <math.h>
}

// Controla el error matematico
extern int ERROR_MATEMATICO;

// Calcula el inverso de X
extern double inv(const double x);

// Indica el tipo de conversion activa el la calculadora
int   Tipo_conversion;                    


///////////////////////////////////////////////////////////////
// Optimizar para no manejar cadenas
// Usar tipo de conversion para funciones trigonometricas
// Hacer remplazo de PI
///////////////////////////////////////////////////////////////



// Rutina que recibe un indicador de operacion asi como un numero determinado
// de operandos para realizar la operacion indicada
double Calcula(const unsigned int operacion, const double op1, const double op2)
{
   double res = 0;
   // operaciones basicas
   switch(operacion) {
     case 1:
        res = op1 * op2;
        break;
     case 2:
        res = op1 * inv(op2);
        break;
     case 3:
        res = op1 + op2;
        break;
     case 4:
        res = op1 - op2;
        break;
     // Funciones trigonometricas
     case 30:   // Seno
        res = sin(op1);
        break;
     case 31:   // Coseno
        res = cos(op1);
        break;
     case 32:   // Tangente
        res = tan(op1);
        break;
     case 33:   // Arco seno
        res = asin(op1);
        break;
     case 34:   // Arco coseno
        res = acos(op1);
        break;
     case 35:   // Arco tangente
        res = atan(op1);
        break;
     case 36:   // Seno hiperbolico
        res = sinh(op1);
        break;
     case 37:   // Coseno hiperbolico
        res = cosh(op1);
        break;
     case 38:   // Tangente hiperbolico
        res = tanh(op1);
        break;
     // Otras funciones
     case 39:   // Valor absoluto
        res = fabs(op1);
        break;
     case 40:   // exponencia e a la x
        res = exp(op1);
        break;
     case 41:   // Logaritmo natural
        res = log(op1);
        break;
     case 42:   // logaritmo base 10
        res = log10(op1);
        break;
     case 43:   // Raiz cuadrada
        res = sqrt(op1);
        break;
     case 44:   // Calcula el modulo de X/Y
        res = fmod(op1,op2);
        break;
     case 45:   // Calcula X  por 2 exponente a la Y
        res = ldexp(op1,op2);
        break;
     case 46:   // X ala Y
        res = pow(op1,op2);
        break;
     case 50:   // Factorial de x
        res = 1;
        {
          for(int i = 2; i <= op1; i++) res *= i;
        }
        break;
     case 51:   // Inverso de x
        res = inv(op1);
        break;
   }
   return res;
}


// Rutina que calcula el valor numerico de una expresion
// en notacion polaca con separador de operandos el pipe
double Calculadora_not_polaca(const char *cadena)
{
   if(!cadena[0]) return 0;
   // Indica el numero maximo de elementos en la pila
   const unsigned Num_max_elem_pila = 40;
   unsigned int pos;
   int i = 0, xi = 0, tp = 0;
   double res = 0, op1 = 0, op2 = 0;
   char xcad[100], numero[100];
   xcad[0] = 0;
////////////////////////////////////////
// Hay que inspeccionar la cadena para saber
// el numero de elementos de la pila
////////////////////////////////////////
   PILA pila(Num_max_elem_pila);
   while(cadena[i+1]) {
      // Almacena los caracteres dentro de la cadena temporal
      if(cadena[i] == ',') i += 2;
      if(cadena[i] != '|') xcad[xi] = cadena[i], xi++, xcad[xi] = 0;
       else {
         xi = tp = 0, op1 = 0, op2 = 0;


         // Funciones que requieren un solo parametro
         if(s_compara(xcad,"sin"))  tp = 30;  // Seno
         if(s_compara(xcad,"cos"))  tp = 31;  // Coseno
         if(s_compara(xcad,"tan"))  tp = 32;  // Tangente
         if(s_compara(xcad,"asin")) tp = 33;  // Arco seno
         if(s_compara(xcad,"acos")) tp = 34;  // Arco coseno
         if(s_compara(xcad,"atan")) tp = 35;  // Arco tangente
         if(s_compara(xcad,"sinh")) tp = 36;  // Seno hiperbolico
         if(s_compara(xcad,"cosh")) tp = 37;  // Coseno hiperbolico
         if(s_compara(xcad,"tanh")) tp = 38;  // Tangente hiperbolico
         if(s_compara(xcad,"abs"))  tp = 39;  // Valor absoluto
         if(s_compara(xcad,"exp"))  tp = 40;  // Exponencia e a la x
         if(s_compara(xcad,"log"))  tp = 41;  // Logaritmo natural
         if(s_compara(xcad,"log10"))tp = 42;  // Logaritmo base 10
         if(s_compara(xcad,"sqrt")) tp = 43;  // Raiz cuadrada
         if(s_compara(xcad,"fact")) tp = 50;  // Factorial de x
         if(s_compara(xcad,"inv"))  tp = 51;  // Inverso de x
         if(tp) {
            pila.Sacar(numero);
            op1 = atof(numero);
            res = Calcula(tp,op1,op2);
            sprintf(xcad,"%4.16f",res);
            pila.Meter(xcad);
            xcad[0] = 0;
            tp = 0;
         }

         // Funciones que requieren dos parametros
         if(s_compara(xcad,"fmod")) tp = 44;  // Calcula el modulo de X/Y
         if(s_compara(xcad,"ldexp")) tp = 45;  // Calcula X  por 2 exponente a la Y
         if(s_compara(xcad,"pow")) tp = 46;  // X a la Y
         if(tp) {
            pila.Sacar(numero);
            op2 = atof(numero);
            pila.Sacar(numero);
            op1 = atof(numero);
            res = Calcula(tp,op1,op2);
            sprintf(xcad,"%4.16f",res);
            pila.Meter(xcad);
            xcad[0] = 0;
            tp = 0;
         }

         // Funciones de operaciones basicas
         if(s_busca_caracter("*+/-",cadena[i+1],pos) && cadena[i+2] == '|') {
            if(xcad[0]) pila.Meter(xcad);
            pila.Sacar(numero);
            op2 = atof(numero);
            pila.Sacar(numero);
            op1 = atof(numero);
            if(cadena[i+1] == '*') tp = 1;   // Producto
            if(cadena[i+1] == '/') tp = 2;   // Division
            if(cadena[i+1] == '+') tp = 3;   // Suma
            if(cadena[i+1] == '-') tp = 4;   // Resta
            res = Calcula(tp,op1,op2);
            sprintf(xcad,"%4.16f",res);
            pila.Meter(xcad);
            xcad[0] = 0;
            i ++, tp = 0;
         }

         // Funciones que no requieren parametros 
//         if(s_compara(xcad,"gra"))  Tipo_conversion = 1;  // Funciones trigonometricas en gradianes
//         if(s_compara(xcad,"rad"))  Tipo_conversion = 2;  // Funciones trigonometricas en radianes
//         if(s_compara(xcad,"deg"))  Tipo_conversion = 3;  // Funciones trigonometricas en grados
         // Descarto la coma como separador de parametros
         if(s_compara(xcad,","))  xcad[0] = 0;

         // Sustitucion de constantes
         if(s_compara(xcad,"M_PI"))       tp = 100, res = M_PI;
         if(s_compara(xcad,"M_PI_2"))     tp = 100, res = M_PI_2;
         if(s_compara(xcad,"M_PI_4"))     tp = 100, res = M_PI_4;
         if(s_compara(xcad,"M_1_PI"))     tp = 100, res = M_1_PI;
         if(s_compara(xcad,"M_2_PI"))     tp = 100, res = M_2_PI;
         if(s_compara(xcad,"M_1_SQRTPI")) tp = 100, res = M_1_SQRTPI;
         if(s_compara(xcad,"M_2_SQRTPI")) tp = 100, res = M_2_SQRTPI;
         if(s_compara(xcad,"M_E"))        tp = 100, res = M_E;
         if(s_compara(xcad,"M_LOG2E"))    tp = 100, res = M_LOG2E;
         if(s_compara(xcad,"M_LOG10E"))   tp = 100, res = M_LOG10E;
         if(s_compara(xcad,"M_LN2"))      tp = 100, res = M_LN2;
         if(s_compara(xcad,"M_LN10"))     tp = 100, res = M_LN10;
         if(tp) {
            sprintf(xcad,"%4.16f",res);
            pila.Meter(xcad);
            xcad[0] = 0;
            tp = 0;
         }

         
         // Introduce el valor en la pila
         if(xcad[0]) {
            pila.Meter(xcad);
            xcad[0] = 0;
         }
         // Signo igual termina el proceso
         if(cadena[i+1] == '=') break;
      }
      i++;
      // Aborta el calculo por error numerico
      if(ERROR_MATEMATICO) return 0.0;
   }
   pila.Sacar(numero);
   return atof(numero);
}









// Evalua una expresion EXP regresando el valor numerico de esta y la cadena con dicho valor en RES
double Evalua_expresion(const char *exp, char *res)
{
   char *exp_fin = new char[strlen(exp)+100];
   char *exp_not_pol = new char[strlen(exp)+100];
   double nres = 0;


   // Acomoda en expresion una variable de asignacion
   sprintf(exp_fin,"x=%s",exp);
   // Realiza el analisis sintactico de la expresion
   Analizador_sintactico *Ev = new Analizador_sintactico;
   Ev->Crea_arbol(exp_fin);
   if(!Ev->Numero_errores()) {
      // Retorna la cadena en notacion polaca
      Ev->Retorna_expresion(exp_not_pol,POSTORDEN);
      // Obtiene el resultado de la expresion polaca
      nres = Calculadora_not_polaca(exp_not_pol);
      // Controla el error matematico
      if(ERROR_MATEMATICO) ERROR_MATEMATICO = 0, nres = 0;

      gcvt(nres,16,res);

///*
      Vis_archivo *Ve = new Vis_archivo;
      Ve->Parametros(" ","AVISO",350,300,VISUALIZA);
      Ve->Adiciona_linea(exp_fin);
      Ve->Adiciona_linea(exp_not_pol);
      Ve->Loop();
      delete Ve;
//*/

   } else {
      // visualiza los errores encontrados dentro de la expresion
      Vis_archivo *Ve = new Vis_archivo;
      Ve->Parametros(" ","AVISO",350,300,VISUALIZA);
      // Visualiza los errores en el analizador
      Ve->Adiciona_linea(exp);
      Ve->Adiciona_linea(" ");
      for(int i = 0; i < Ev->Numero_errores(); i++) Ve->Adiciona_linea(Ev->Retorna_error(i));
      Ve->Loop();
      delete Ve;
   }
   delete []exp_fin;
   delete []exp_not_pol;
   delete Ev;
   return nres;
}




