// Autor:     Antonio Carrillo Ledesma.
// R.F.C.:    CALA-691229-TV5
// Direccin: Amsterdam 312 col. Hipodromo Condesa
// Telfono:  5-74-43-53

// Propiedad intelectual, todos los derechos reservados conforme a la ley, registro en trmite
// Revisin  1.1-A


#ifndef __V_GRAFIC_HPP__
#define __V_GRAFIC_HPP__


#include "v_iconos.hpp"
#include "definic.hpp"

extern "C" {
   #include <graphics.h>
}

// Indica el maximo numero de pixes horizontales hay en ambiente grafico
extern int X_MAX;
// Indica el maximo numero de pixes verticales hay en ambiente grafico
extern int Y_MAX;



/////////////////////////////////////////////////////////////
//                                                         //
//                    Ventana de graficacion               //
//                                                         //
/////////////////////////////////////////////////////////////

class Ventana_graficacion: public Ventana_iconos
{
   private:

      double        Factor;                      // Factor de acercamiento dentro de la ventana


                    // Revisa que la sona de graficacion este dentro de la ventana
      virtual int   Revisa_graficacion(void) const  = 0;

   protected:

      C_3D          Escala;                      // Escala en X, Y, Z
      C_2I          Centro;                      // Centro de la pantalla
      C_3D          Minimo;                      // Contiene los valores de X, Y, Z minimos de los ejes
      C_3D          Maximo;                      // Contiene los valores de X, Y, Z maximos de los ejes
      C_3D          C_reales;                    // Posicion dentro de los ejes coordenados (reales)
//      C_2I          V_pos_ini;                   // Indica la posicion donde iniciara la visualizacion de la ventana
//      C_2I          V_pos_fin;                   // Indica la posicion donde finaliza la visualizacion de la ventana

      double        TETA, FI;                    // Valores usados para la proyeccin
      double        ST, SF, CT, CF;              // Valores usados para la proyeccin
      unsigned int  Proyeccion:1;                // Indica la proyeccion actual
//      unsigned int  Tipo_ventana:1;              // Indica el tipo de ventan a visualizar

      double        x1,x2,x3,x4;                 // Valores temporales
      double        pt_x, pt_y;                  // Valores temporales para proyeccion
      C_ptl         aux0;                        // Valores temporales
      C_2I          aux1,aux2;
      C_3D          aux3,aux4;


                    // Fija escalas
      void          Fija_escalas(void) const;
                    // Calcula valores para proyecciones
      void          Calcula_valores(void) const;

   public:

                    // Inicializa la clase
      virtual void  Inicializa(const int t_proy, const double factor) const;

                    // Dibuja al ventana
      virtual void  Dibuja(void) const
                    {
                       Ventana_iconos::Dibuja();
                    }

                    // Modifica la dimencion de los ejes coordenados
      virtual void  Fija_dimenciones(const C_3D v_max, const C_3D v_min) const
                    {
                       ((Ventana_graficacion*)this)->Maximo = v_max, ((Ventana_graficacion*)this)->Minimo = v_min;
                       Fija_escalas();
                    }

                    // Retorna la dimencion de los ejes coordenados
      void          Retorna_dimenciones(C_3D &v_max,C_3D &v_min) const
                    {
                       v_max = Maximo,v_min =  Minimo;
                    }

                    // Modifica coordenadas reales
      virtual void  Fija_coordenadas(const C_3D n_cda) const
                    {
                       ((Ventana_graficacion*)this)->C_reales = n_cda;
                    }

                    // Retorna el valor de coordenadas reales
      C_3D          Retorna_coordenadas(void) const
                    {
                       return C_reales;
                    }

                    // Fija la proyeccion actual
      void          Fija_proyeccion_actual(const int proy) const
                    {
                       ((Ventana_graficacion*)this)->Proyeccion = proy;
                    }

                    // Retorna la proyeccion actual
      int           Retorna_proyeccion_actual(void) const
                    {
                       return Proyeccion;
                    }

                    // Retorna la posicion del mouse dentro de la ventana
      virtual int   Posicion_mouse(C_2D &pos) const;


                    // Mueve el origen coordenado
      virtual void  Mover_origen(const int tp) const;
                    // Acercar alejar el origen
      virtual void  Acercar_alejar(const char tp) const;
                    // Acercamiento 
      void          Acercamiento(void) const;

};



/////////////////////////////////////////////////////////////
//                                                         //
//        Ventana de graficacion en dos dimenciones        //
//                                                         //
/////////////////////////////////////////////////////////////

class Ventana_graficacion_2D: virtual public Ventana_graficacion
{

   public:

                    // Calcula la proyeccion de un punto de 2D a la ventana
              int   Proyecciones_2D(const C_3D pt_r, C_2I &pt_p) const
                    {
                       ((Ventana_graficacion_2D*)this)->pt_x = (pt_r.x * Escala.x) + Centro.x;
                       ((Ventana_graficacion_2D*)this)->pt_y =  Centro.y - (pt_r.y * Escala.y);
                       if(pt_x > 32000.00 || pt_x < -32000 || pt_y > 32000.00 || pt_y < -32000) return 0;
                       pt_p.x = (int) pt_x;
                       pt_p.y = (int) pt_y;
                       if(pt_p.x > (X1 + Retorna_inicio_X()) &&  pt_p.x < (X1 + Largo_X() -3) && pt_p.y > (Y1 + Retorna_inicio_Y()+2) && pt_p.y < (Y1 + Largo_Y() - 3)) return 0;
                       return 1;
                    }
     
                    // Dibuja un punto de 2D dentro de la ventana
              void  Dibuja_punto_2D(const C_3D cd, const int cl) const
                    {
                        if(Proyecciones_2D(cd,((Ventana_graficacion_2D*)this)->aux1)) return;
                        setcolor(cl);
                        putpixel(aux1.x,aux1.y,cl);
                    }
                    // Dibuja una linea con las coordenadas reales dadas con el color especificado
              void  Dibuja_linea_2D(const C_3D ini, const C_3D fin, const int cl) const
                    {
                        if(Proyecciones_2D(ini,((Ventana_graficacion_2D*)this)->aux1)) return;
                        if(Proyecciones_2D(fin,((Ventana_graficacion_2D*)this)->aux2)) return;
                        setcolor(cl);
                        line(aux1.x,aux1.y,aux2.x,aux2.y);
                    }

                    // Rota los los ejes coordenados
//      virtual void  Rota_ejes(const int tp) const;
};




/////////////////////////////////////////////////////////////
//                                                         //
//       Ventana de graficacion en tres dimenciones        //
//                                                         //
/////////////////////////////////////////////////////////////

class Ventana_graficacion_3D: virtual public Ventana_graficacion
{

   public:

                    // Calcula la proyeccion de un punto de 3D a la ventana
              int   Proyecciones_3D(const C_3D pt_r, C_2I &pt_p) const
                    {
                       ((Ventana_graficacion_3D*)this)->aux3 = pt_r;
                       ((Ventana_graficacion_3D*)this)->aux3.x  *= Escala.x, ((Ventana_graficacion_3D*)this)->aux3.y *= Escala.y, ((Ventana_graficacion_3D*)this)->aux3.z *= Escala.z;
                       ((Ventana_graficacion_3D*)this)->aux4.x = (aux3.z * ST) + (aux3.x * CT);
                       ((Ventana_graficacion_3D*)this)->aux4.y = (aux3.z * x1) - (aux3.x * x2) + (aux3.y * SF);
                       ((Ventana_graficacion_3D*)this)->aux4.z = (aux3.z * x3) - (aux3.x * x4) - (aux3.y * CF) + 770;
                       ((Ventana_graficacion_3D*)this)->pt_x = ((800 * aux4.x) / aux4.z) + Centro.x;
                       ((Ventana_graficacion_3D*)this)->pt_y = Centro.y - ((800 * aux4.y) / aux4.z) + 5;

                       if(pt_x > 32000.00 || pt_x < -32000 || pt_y > 32000.00 || pt_y < -32000) return 0;
                       pt_p.x = (int) pt_x;
                       pt_p.y = (int) pt_y;

                       if(pt_p.x > (X1 + Retorna_inicio_X()) &&  pt_p.x < (X1 + Largo_X() -3) && pt_p.y > (Y1 + Retorna_inicio_Y()+2) && pt_p.y < (Y1 + Largo_Y() - 3)) return 0;
                       return 1;
                    }
              
                    // Dibuja un punto de 3D dentro de la ventana
              void  Dibuja_punto_3D(const C_3D cd, const int cl) const
                    {
                       if(Proyecciones_3D(cd,((Ventana_graficacion_3D*)this)->aux1)) return;
                       setcolor(cl);
                       putpixel(aux1.x,aux1.y,cl);
                    }
   
                    // Dibuja una linea con las coordenadas reales dadas con el color especificado
              void  Dibuja_linea_3D(const C_3D ini, const C_3D fin, const int cl) const
                    {
                       if(Proyecciones_3D(ini,((Ventana_graficacion_3D*)this)->aux1)) return;
                       if(Proyecciones_3D(fin,((Ventana_graficacion_3D*)this)->aux2)) return;
                       setcolor(cl);
                       line(aux1.x,aux1.y,aux2.x,aux2.y);
                    }

                    // Rota los los ejes coordenados
      virtual void  Rota_ejes(const int tp) const;
};


/////////////////////////////////////////////////////////////
//                                                         //
// Ventana con cursor                                      //
//                                                         //
/////////////////////////////////////////////////////////////

class V_cursor: virtual public Ventana_graficacion_2D, virtual public Ventana_graficacion_3D
{
   private:

      char         *Cursor;            // Puntero que contiene la imagen abajo del cursor
      C_2I          Pos_ant;           // Ultimo punto donde se visualizo el cursor
      unsigned int  sw_cursor:1;


   protected:
                    // Inicializa la clase

      virtual void  Inicializa(void) const;

                    // Destruye la clase
      virtual void  Destruye(void) const
                    {
                       delete []Cursor;
                    }

   public:
     
                    // Visualiza el cursor tridimencional
      void          Visualiza_cursor(const int tp) const;

};




/////////////////////////////////////////////////////////////
//                                                         //
//         Ventana con ejes coordenados en 2D y 3D         //
//                                                         //
/////////////////////////////////////////////////////////////

class V_ejes_coordenados: virtual public Ventana_graficacion_2D, virtual public Ventana_graficacion_3D
{
   private:

      char           Nombre_ejes[4];             // Cadena que contiene el nombre de los ejes coordenados
//      char *Nombre_eje_X;                        // Nombre del eje X
//      char *Nombre_eje_Y;                        // Nombre del eje Y
//      char *Nombre_eje_Z;                        // Nombre del eje Z

      C_3I           Color_eje;                  // Contiene los colores para los ejes XYZ
      unsigned int   Visualiza_nombre_eje:1;     // Indica si visualiza el nombre del eje coordenado

   protected:
                      // Dibuja ejes coordenados en 2D
      void            Dibuja_ejes_2D(const int tp) const;
                      // Dibuja ejes coordenados en 3D
      void            Dibuja_ejes_3D(const int tp) const;

   public:
                      // Inicializa la clase
      virtual void    Inicializa(const int t_proy, const double factor, const char *n_ejes) const;

                      // Dibuja los ejes coordenados
              void    Dibuja_ejes_coordenados(const int tp) const
                      {
                         if(Proyeccion) Dibuja_ejes_2D(tp);
                          else Dibuja_ejes_3D(tp);
                      }
                      // Gradua los ejes coordenados
      void            Gradua_ejes(void) 
                      {
          // No esta completo el procedimiento
//          C_3D ini;
//          ini.x = 0, ini.y = 0, ini.z = 0;
//          for(ini.x = Dim_minima[Sistema_actual].x; ini.x < Dim_maxima[Sistema_actual].x; ini.x +=(Dim_maxima[Sistema_actual].x-Dim_minima[Sistema_actual].x)/20) Dibuja_punto(ini,0,Magenta9);
//          ini.x = 0;
//          for(ini.y = Dim_minima[Sistema_actual].y; ini.y < Dim_maxima[Sistema_actual].y; ini.y +=(Dim_maxima[Sistema_actual].y-Dim_minima[Sistema_actual].y)/20) Dibuja_punto(ini,0,Magenta9);
//          ini.y = 0;
//          for(ini.z = Dim_minima[Sistema_actual].z; ini.z < Dim_maxima[Sistema_actual].z; ini.z +=(Dim_maxima[Sistema_actual].z-Dim_minima[Sistema_actual].z)/20) Dibuja_punto(ini,0,Magenta9);
                      }
                      // Retorna el estado de visualiza nombre de eje coordenado
      int             Retorna_Visualiza_nombre_eje(void) const
                      {
                         return Visualiza_nombre_eje;
                      }
                      // Fija el estado de visualiza nombre de eje coordenado
      void            Fija_Visualiza_nombre_eje(const int tp) const
                      {
                         ((V_ejes_coordenados*)this)->Visualiza_nombre_eje = tp;
                      }
                      // Fija el nombre de los ejes coordenado
      void            Fija_nombre_ejes(const char *cad) const
                      {
                         ((V_ejes_coordenados*)this)->Nombre_ejes[0] = cad[0], ((V_ejes_coordenados*)this)->Nombre_ejes[1] = cad[1], 
                         ((V_ejes_coordenados*)this)->Nombre_ejes[2] = cad[2], ((V_ejes_coordenados*)this)->Nombre_ejes[3] = cad[3];
                      }
                      // Fija el color de los ejes coordenado
      void            Fija_color_ejes(const C_3I color) const
                      {
                         ((V_ejes_coordenados*)this)->Color_eje = color;
                      }
                      // Retorna el color de los ejes coordenado
      void            Retorna_color_ejes(C_3I &color) const
                      {
                         color = Color_eje;
                      }

};


///////////////////////////////////////////////////////////////
//                                                           //
// Ventana de graficacion en 2D y 3D con cursor y ejes       //
// cordenados                                                //
//                                                           //
///////////////////////////////////////////////////////////////

class V_graficacion: virtual public V_ejes_coordenados, virtual public V_cursor
{

   private:

                  // Revisa que la zona de graficacion este dentro de la ventana
      int         Revisa_graficacion(void) const;

   public:
                     // Inicializa la clase
      void        Inicializa(const int t_proy, const double factor, const char *n_ejes) const
                  {
                     V_ejes_coordenados::Inicializa(t_proy,factor,n_ejes);
                     V_cursor::Inicializa();
                  }
                      // Destruye la clase
      void        Destruye(void) const
                  {V_cursor::Destruye();}


                  // Dibuja un punto
      void        Dibuja_punto(const C_3D aux, const char col) const
                  {
                     if(Proyeccion) Dibuja_punto_2D(aux,col);
                      else Dibuja_punto_3D(aux,col);
                  }
                  // Dibuja una linea
      void        Dibuja_linea(const C_3D aux1,const C_3D aux2, const char col) const
                  {
                     if(Proyeccion) Dibuja_linea_2D(aux1,aux2,col);
                      else Dibuja_linea_3D(aux1,aux2,col);
                  }

                  // Rota los ejes coordenados
      void        Rota_ejes(const int tp) const
                  {
                   if(!Proyeccion) {
                      Visualiza_cursor(0);
                      Dibuja_ejes_coordenados(0);
                      Ventana_graficacion_3D::Rota_ejes(tp);
                      Dibuja_ejes_coordenados(1);
                      Visualiza_cursor(1);
                   }
                  }
                  // Mueve el origen de los ejes coordenados
      void        Mover_origen(const int tp) const
                  {
                   Visualiza_cursor(0);
                   Dibuja_ejes_coordenados(0);
                   Ventana_graficacion::Mover_origen(tp);
                   Dibuja_ejes_coordenados(1);
                   Visualiza_cursor(1);
                  }
                  // Acerca o aleja al origen de los ejes coordenados
      void        Acercar_alejar(const int tp) const
                  {
                   Visualiza_cursor(0);
                   Dibuja_ejes_coordenados(0);
                   Ventana_graficacion::Acercar_alejar(tp);
                   Dibuja_ejes_coordenados(1);
                   Visualiza_cursor(1);
                  }
                  // Dibuja los ejes coordenados
      void        Dibuja(void) const
                  {
                   Ventana_graficacion::Dibuja();
                   Dibuja_ejes_coordenados(1);
                   Visualiza_cursor(1);
                  }
      void        Fija_coordenadas(const C_3D n_cda) const
                  {
                   Visualiza_cursor(0);
                   V_ejes_coordenados::Fija_coordenadas(n_cda);
                   Visualiza_cursor(1);
                  }
                  // Revisualiza los ejes coordenados (Sobrepone los ejes)
      void        Revisualiza_ejes(void) const
                  {
                   Visualiza_cursor(0);
                   Dibuja_ejes_coordenados(1);
                   Visualiza_cursor(1);
                  }
/*
      void        Cambiar_tamano(const int x1, const int y1, const int x2, const int y2) const
                  {
                   V_ejes_coordenados::Cambiar_tamano(x1,y1,x2,y2);
                   Fija_escalas();
                   Dibuja();
                  }
*/
///////////////////////////////
// Optimizar la configuracion 
                  // Retorna la actual configuracion de la ventana
      void        Retorna_configuracion(C_3D &esc, C_2I &cen, double &teta, double &fi) const
                  {  
                     esc = Escala, cen = Centro;
                     teta = TETA,
                     fi = FI;
                  }

                  // Fija nueva configuracion de la ventana
      void        Fija_configuracion(const C_3D esc,const C_2I cen, const double teta, const double fi) const
                  {
                     ((V_graficacion*)this)->Escala = esc,
                     ((V_graficacion*)this)->Centro = cen;
                     ((V_graficacion*)this)->TETA = teta,
                     ((V_graficacion*)this)->FI = fi;
                     Calcula_valores();
                  }

};


class Ctrl_grafico {

   private:

      V_graficacion   *VT1;
      V_graficacion   *VT2;
      int              Tipo_ventana;

   public:

      void Inicializa(const int tipo, const char ejes[][15], const char *tit)
           {
              Tipo_ventana = tipo;
              VT1 = new V_graficacion;
              VT1->Activa_color_refresco(Blanco);
              VT1->Nombre_iconos("MAXIM.ICO","MINIM.ICO");
              VT1->Define_colores(Blanco,Cian9,Gris9,Blanco);
              VT1->Define_ventana(tit,3,83,X_MAX-3,Y_MAX-31,1);
              VT1->Inicializa(0,0.80,"XYZ");

              VT2 = new V_graficacion;
              VT2->Activa_color_refresco(Blanco);
              VT2->Nombre_iconos("MAXIM.ICO","MINIM.ICO");
              VT2->Define_colores(Blanco,Cian9,Gris9,Blanco);
              VT2->Define_ventana(tit,3,83,X_MAX-3,Y_MAX-31,1);
              VT2->Inicializa(0,0.80,"XYZ");
           }


      void Destruye(void)
           {
              VT1->Destruye();
              VT2->Destruye();
           }

           // Dibuja la ventana
      void Dibuja(void)
           {
              VT1->Dibuja();
              VT2->Dibuja();
           }
           // Dibuja un punto
      void Dibuja_punto(const C_3D aux, const char col)
           {
              VT1->Dibuja_punto(aux,col);
              VT2->Dibuja_punto(aux,col);
           }
           // Dibuja una linea
      void Dibuja_linea(const C_3D aux1,const C_3D aux2, const char col)
           {
              VT1->Dibuja_linea(aux1,aux2,col);
              VT2->Dibuja_linea(aux1,aux2,col);
           }
           // Fija nueva configuracion de la ventana
      void Fija_configuracion(const C_3D esc,const C_2I cen, const double teta, const double fi)
           {
              VT1->Fija_configuracion(esc,cen,teta,fi);
              VT2->Fija_configuracion(esc,cen,teta,fi);
           }
           // Modifica la dimencion de los ejes coordenados
      void Fija_dimenciones(const C_3D v_max, const C_3D v_min)
           {
              VT1->Fija_dimenciones(v_max,v_min);
              VT2->Fija_dimenciones(v_max,v_min);
           }
      void Retorna_configuracion(C_3D &esc, C_2I &cen, double &teta, double &fi)
           {
             if(Tipo_ventana == 2) VT2->Retorna_configuracion(esc,cen,teta,fi);
              else VT1->Retorna_configuracion(esc,cen,teta,fi);
           }
                  // Revisa si el boton es oprimido por el mouse dentro de la ventana
   unsigned  int Presionado(const int boton)
           {
              if(Tipo_ventana == 2) return VT2->Presionado(boton);
              return VT1->Presionado(boton);
           }
   void    Fija_color_ejes(const C_3I color)
           {
              VT1->Fija_color_ejes(color);
              VT2->Fija_color_ejes(color);
           }
   void    Define_titulo(const char *tit)
           {
              VT1->Define_titulo(tit);
              VT2->Define_titulo(tit);
           }
   void    Fija_proyeccion_actual(const int proy)
           {
              VT1->Fija_proyeccion_actual(proy);
              VT1->Fija_proyeccion_actual(proy);
           }

   void    Fija_nombre_ejes(const char *cad)
           {
              VT1->Fija_nombre_ejes(cad);
              VT2->Fija_nombre_ejes(cad);
           }
  void  Fija_coordenadas(const C_3D n_cda)
        {
           VT1->Fija_coordenadas(n_cda);
           VT1->Fija_coordenadas(n_cda);

        }

};

#endif

